#!/usr/bin/python3
# -*- coding: utf-8 -*-
import sys
import apt
import dbus
import dbus.service
import dbus.mainloop.glib
import os
import time
import datetime
import shutil
import threading
import re
import apt_pkg
from apt.debfile import DebPackage
from kylindbus.apt_handler import AppActions, AptHandler, WorkitemError
from kylindbus.enums import INTERFACE, INTERFACES, check_apt_lock
from kylindbus.policykit_service import PolicyKitService
from kylindbus.log_manager import logger
from kylindbus.sqlite3_server import Sqlite3Server
from PyQt5.QtCore import pyqtSignal
from kylindbus.log_message_server import ConnectServerThread
CRONPATH = "/var/spool/anacron/"
SOURCESPATH = "/etc/apt/sources.list.d"
CODEHOME = "/usr/share/kylin-update-manager/"
PACKAGES_MANAGER_TOOLS = 'cn.kylinos.KylinUpdateManager.action'

class WorkItem:
     def __init__(self, pkgname, action, kwargs):
        self.pkgname = pkgname
        self.action = action
        self.kwargs = kwargs

class WorkThread(threading.Thread):
    def __init__(self, dbusDaemon):
        threading.Thread.__init__(self)
        self.dbusDaemon = dbusDaemon
        ("KUM DBUS START ", type(self.dbusDaemon))

    #
    # 函数:线程执行函数
    #
    def run(self):
        while(True):
            if len(self.dbusDaemon.worklist) == 0:
                time.sleep(0.2)
                continue
            self.dbusDaemon.mutex.acquire()
            self.item = self.dbusDaemon.worklist.pop(0)
            self.dbusDaemon.mutex.release()
            try:
                func = getattr(self.dbusDaemon.aptHandler, self.item.action)
                if func is None:
                    print("Error action: ", self.item)
                res = func(self.item.pkgname,self.item.kwargs)
                if res is False:
                    print("Action exec failed...")
            except WorkitemError as e:
                # 如果是包列表，此时强制装换为str，后面显示str将出现异常，故而暂不显示包列表
                kwarg = {"apt_appname": str(self.item.pkgname),
                        "apt_percent": str(e.errornum),
                        "error_message": str(e.details),
                        }
                #安装失败时需要进行二次判断，防止第一次安装失败导致后续的包都安装失败
                result = ""
                results = os.popen("dpkg -l | grep %s"%self.item.pkgname).readlines()
                for res in results:
                    res = re.sub(" +", " ", res)
                    if str(res).split(" ")[1] == self.item.pkgname:
                        result = res.split(" ")[0]
                if result != "ii":
                    logger.error("安装状态：失败 - 包名：" + str(self.item.pkgname) + " - 错误码：(" + str(e.errornum) +") - 详情：" + str(e.details))
                    self.dbusDaemon.kum_apt_signal("apt_error", kwarg)
                    self.dbusDaemon.insert_app_message("apt_error", kwarg)
            except Exception as e: #代码异常
                kwarg = {"apt_appname": str(self.item.pkgname),
                        "apt_percent": '24',
                        "error_message": str(e),
                        }
                logger.error("安装状态：失败 - 包名：" + str(self.item.pkgname) + " - 错误码：(24) - 详情：" + str(e), exc_info=True)
                    # self.dbusDaemon.kum_apt_signal("apt_error", kwarg)
                    # self.dbusDaemon.insert_app_message("apt_error", kwarg)
            finally:
                self.dbusDaemon.error_message = ""
                self.item = None
                self.dbusDaemon.counter -= 1
            time.sleep(0.3)

class Daemon(PolicyKitService):
    def __init__ (self, bus, mainloop):
        self.aptHandler = AptHandler(self)
        self.sqlite_server = Sqlite3Server()
        self.worklist = []
        self.cancel_name_list = []
        self.mutex = threading.RLock()
        self.cancelmutex = threading.RLock()
        self.worker_thread = WorkThread(self)
        self.worker_thread.setDaemon(True)
        self.worker_thread.start()
        self.importantwork = False
        self.efi_status = ""
        self.cache = apt.Cache()
        self.optional_message_list = []
        self.silent_message_list = []
        self.bus_name = dbus.service.BusName(INTERFACE, bus=bus)
        PolicyKitService.__init__(self, self.bus_name, INTERFACES)
        self.counter = 0
        self.mainloop = mainloop
        self.error_message = ""

    #
    # 函数：检测计数器是否为0，是否可以退出dbus
    #
    def check_counter(self):
        if(self.counter == 0):
            logger.info("自我升级完成，定时自动重启服务")
            self.mainloop.quit()
        else:
            exit_timer = threading.Timer(5, self.check_counter)
            exit_timer.start()

    #
    # 函数：apt安装结果写入数据库
    #
    def insert_app_message(self, type_msg, msg):
        try:
            self.insert_to_sqlite(type_msg, msg, self.optional_message_list, 1)
            self.insert_to_sqlite(type_msg, msg, self.silent_message_list, 0)
        except Exception as e: #数据库文件被删除或者新增字段导致需要重新初始化数据库再写入
            self.sqlite_server.init_sqlit()
            self.insert_to_sqlite(type_msg, msg, self.optional_message_list, 1)
            self.insert_to_sqlite(type_msg, msg, self.silent_message_list, 0)
            logger.error("数据库写入异常，包名：%s"%msg["apt_appname"],exc_info=True)

    #
    # 函数：插入app信息到数据库并发送信号
    #
    def insert_to_sqlite(self,type_msg, msg, list, var = 1):
        time = datetime.datetime.now()
        timestr = datetime.datetime.strftime(time, "%Y-%m-%d %H:%M:%S")
        n = 0
        for i in list:
            if i["appname"] == msg["apt_appname"]:
                appmsg = list.pop(n)
                icon = None
                if type_msg == "apt_finish":
                    statue = "Success"
                    description = appmsg["description"]
                else:
                    statue = "Fail"
                    description = msg["error_message"]
                if msg["apt_percent"] == "100":
                    errorcode = "0"
                else:
                    errorcode = msg["apt_percent"]
                if var == 1:
                    server_errorcode = 10010000 + int(errorcode)
                else:
                    server_errorcode = 10000000 + int(errorcode)
                self.sqlite_server.insert_into_installed(appmsg["appname"], appmsg["source_version"],
                                                         timestr, description, icon,
                                                         statue, i["keyword"], errorcode)
                self.sqlite_server.insert_into_display('update_time', timestr) #写入本地数据库
                if i["keyword"] == "1": #可选更新才发送信号用于更新历史界面
                    self.update_sqlite_signal(appmsg["appname"], timestr)
                self.send_message_to_server(appmsg["appname"], appmsg["source_version"], statue, str(server_errorcode)) #发送到日志服务器
                logger.info("安装状态成功写入数据库，包名：%s" % msg["apt_appname"])
            n += 1

    #
    # 函数：插入deb本地包安装信息到数据库
    #
    def insert_deb_message(self, kum, msg):
        time = datetime.datetime.now()
        timestr = datetime.datetime.strftime(time, "%Y-%m-%d %H:%M:%S")
        if msg["statue"] == "Fail":
            description = msg["error_message"]
        else:
            description = kum["description"]
        try:
            self.sqlite_server.insert_into_installed(kum["appname"], kum["version"],
                                                    timestr, description, kum["icon"],
                                                    msg["statue"], kum["keyword"], msg["errorcode"])
        except:
            self.sqlite_server.init_sqlit()
            self.sqlite_server.insert_into_installed(kum["appname"], kum["version"],
                                                     timestr, description, kum["icon"],
                                                     msg["statue"], kum["keyword"], msg["errorcode"])

    #
    # 函数:新增任务
    # item：任务
    #
    def add_worker_item(self, item):
        print("####add_worker_item:", item)
        self.mutex.acquire()
        self.worklist.append(item)
        self.mutex.release()

    #
    # 函数：获取当前下载包的依赖
    #
    def get_deps_pkg(self, appname):
        url_list = []
        name_list = []
        size_list = []
        fullname_list = []
        all_size = 0
        depends_status = False
        try:
            res = os.popen("LANG= apt-get install --print-uris %s -y" % appname).readlines()
            # print(len(res))
            num = 0
            for i in res:
                if ("After this operation," in i):
                    num = num + 1
                    depends_status = True
                    while (num < len(res)):
                        i = res[num]
                        i_list = i.strip("\n").split(" ")
                        url_list.append(i_list[0].strip("'"))
                        name_list.append(i_list[1].split("_")[0])
                        fullname_list.append(i_list[1])
                        size_list.append(i_list[2])
                        all_size = all_size + int(i_list[2])
                        num = num + 1
                    break
                # 软件包已安装(kylin-update-desktop单独处理)-lj
                elif ("0 upgraded," in i and " 0 newly" in i):
                    # print("len:",len(res), "num:", num+1)
                    if len(res) == num + 1:
                        depends_status = True
                        break
                elif ("The following packages have unmet dependencies:" in i):
                    logger.error("--%s获取依赖失败--"%(appname) + "\n" + "".join(res))
                    depends_status = False
                    break
                num = num + 1
            return url_list, name_list, fullname_list, size_list, all_size, depends_status
        except Exception as e:
            logger.error("依赖解析代码出错--%s--"%appname,exc_info=True)
            return url_list, name_list, fullname_list, size_list, all_size, depends_status

    #
    # 函数：判断efi分区的状态
    #
    def get_efi_status(self):
        with open("/proc/mounts") as f:
            for line in f.readlines():
                line_section = line.split(" ")
                if "/boot/efi" in line_section[1]:
                    status = line_section[3].split(",")
                    for i in status:
                        if i == "ro" or i == "rw":
                            return i
                    break

    #
    # 函数：安装前后设置efi分区的状态为读写
    #
    def set_efi_status(self, before=True):
        try:
            if(self.efi_status == "ro" and before == True):
                os.system("mount -o remount,rw /boot/efi")
                os.system("mount -o remount,rw /boot")
            elif(self.efi_status == "ro" and before == False):
                os.system("mount -o remount,ro /boot/efi")
                os.system("mount -o remount,ro /boot")
        except Exception as e:
            logger.error("修改/boot/efi挂载状态出错",exc_info=True)

    #
    # 函数：开启线程收集客户端信息发送给php服务器
    #
    def send_message_to_server(self,_appname = "" , _appversion = "" , _statue = "", _errorcode = ""):
        send_thread = ConnectServerThread(self)
        send_thread.get_values(_appname, _appversion, _statue, _errorcode)
        send_thread.start()

    #
    # 函数：共用函数用于设置apt状态并发送信号。
    #
    def send_apt_status(self,name,percent,message,status=True):
        if status:
            kwarg = {"apt_appname": str(name), "apt_percent": str(percent), "action": str(message)}
            self.kum_apt_signal("apt_finish", kwarg)
        else:
            kwarg = {"apt_appname": str(name), "apt_percent": str(percent), "error_message": str(message)}
            self.kum_apt_signal("apt_error", kwarg)
        self.set_efi_status(False)
        self.counter -= 1

    #
    # 函数:初始化cache
    #
    def init_cache(self):
        self.aptHandler.init_cache()

    # -------------------------------------dbus methods----------------------------------------

    #
    # 函数:安装和升级
    # install package sa:kum_fetch_signal() and kum_apt_signal()
    #
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    def install_and_upgrade(self, pkgName, sender=None):
        logger.debug("####install_and_upgrade: ", pkgName)
        self.counter += 1
        item = WorkItem(pkgName, AppActions.INS_AND_UP, None)

        self.add_worker_item(item)

        logger.debug("####install_and_upgrade return")
        return True

    #
    # 函数：创建root目录
    #
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    def makedirs(self, path, sender=None):
        self.counter += 1
        try:
            os.makedirs(path)
            self.counter -= 1
            return True
        except:
            self.counter -= 1
            return False

    #
    # 函数：判断包是否可以安装或可升级
    #
    @dbus.service.method(INTERFACE, in_signature='as', out_signature='as', sender_keyword='sender')
    def check_installed_or_upgrade(self, list, sender=None):
        self.counter += 1
        pkglist = []
        sizelist = []
        self.init_cache()
        for i in list:
            try:
                name = str(i).strip().replace("\n","")
                pkg = self.aptHandler.get_pkg_by_name(name)
                if (pkg.is_installed == False or pkg.is_upgradable == True):
                    kum = {"appname": str(pkg.name),
                           "source_version": str(pkg.versions[0]).split("=")[1],
                           "size": str(pkg.candidate.size),
                           "description": str(pkg.candidate.raw_description),
                           "keyword": str(0)}
                    for message in self.silent_message_list:
                        if message["appname"] == kum["appname"]:
                            self.silent_message_list.remove(message)
                            break
                    self.silent_message_list.append(kum)
                    pkglist.append(str(pkg.name))
                    sizelist.append(str(pkg.candidate.size))
            except:
                logger.error("获取关键更新包信息出错,包名：%s" % str(i), exc_info=True)
                continue
        pkglist.extend(sizelist)
        self.counter -= 1
        return pkglist

    #
    # 函数：获取包的信息
    #
    @dbus.service.method(INTERFACE, in_signature='as', sender_keyword='sender')
    def get_app_message(self, list, sender=None):
        self.counter += 1
        self.init_cache()
        # self.optional_message_list.clear()
        for i in list:
            name = str(i).strip().replace("\n", "")  # 防止包名传入\n和空格
            try:
                try:  # 异常处理，防止正在安装包时重新打开控制面依然显示该包
                    work_pkgname = self.worker_thread.item.pkgname
                    if name == work_pkgname:
                        continue
                except:
                    pass
                pkg = self.aptHandler.get_pkg_by_name(name)
                url_list, name_list, size_list, fullname_list, all_size, depends_status = self.get_deps_pkg(pkg.name)
                if (pkg.is_installed == False):
                    kum = {"appname": str(pkg.name),
                           "current_version": "",
                           "source_version": str(pkg.versions[0]).split("=")[1],
                           "size": str(pkg.candidate.size),
                           "description": str(pkg.candidate.raw_description),
                           "depends_status": str(depends_status),
                           "keyword": str(1)}
                    for message in self.optional_message_list:
                        if message["appname"] == kum["appname"]:
                            self.optional_message_list.remove(message)
                            break
                    self.optional_message_list.append(kum)
                    self.important_app_message_signal(kum, url_list, name_list, size_list,
                                                      fullname_list, str(all_size), depends_status)
                elif (pkg.is_upgradable == True):
                    kum = {"appname": str(pkg.name),
                           "current_version": str(pkg.installed.version),
                           "source_version": str(pkg.versions[0]).split("=")[1],
                           "size": str(pkg.candidate.size),
                           "description": str(pkg.candidate.raw_description),
                           "depends_status": str(depends_status),
                           "keyword": str(1)}
                    for message in self.optional_message_list:
                        if message["appname"] == kum["appname"]:
                            self.optional_message_list.remove(message)
                            break
                    self.optional_message_list.append(kum)
                    self.important_app_message_signal(kum, url_list, name_list, size_list,
                                                      fullname_list, str(all_size), depends_status)
            except Exception as e:
                logger.error("获取重要更新包信息出错,包名：%s" % str(name))
                continue
        self.get_message_finished_signal(str(len(self.optional_message_list)))
        self.counter -= 1

    #
    # dbus接口：向数据库中插入数据
    #
    @dbus.service.method(INTERFACE, in_signature='ss', sender_keyword='sender')
    def insert_install_state(self, item, value, sender = None):
        self.counter += 1
        try:
            self.sqlite_server.insert_into_display(item, value)
            self.counter -= 1
            logger.info("数据库状态表插入成功，选项：%s，值：%s"%(item, value))
        except Exception as e:
            self.counter -= 1
            logger.error("数据库状态表插入失败，选项：%s，值：%s"%(item,value))

    #
    # dbus接口：判断包是否可以安装或可升级
    #
    @dbus.service.method(INTERFACE,out_signature='b', sender_keyword='sender')
    def clear_install_list(self, sender=None):
        self.counter += 1
        try:
            self.mutex.acquire()
            self.worklist.clear()
            self.mutex.release()
            self.counter -= 1
            logger.info("成功清理下载列表")
            return True
        except Exception as e:
            logger.error("清理下载列表失败",exc_info=True)
            self.counter -= 1
            return False

    #
    # dbus接口：wget下载软件包后拷贝到/var/cache/apt/archives目录下
    #
    @dbus.service.method(INTERFACE, in_signature='ass', out_signature='')
    def copy_file_to_install(self,src_list,appname):
        self.counter += 1
        for i in src_list:
            if os.path.exists(os.path.join("/var/cache/apt/archives", os.path.basename(i))):
                pass
            else:
                shutil.move(i, os.path.join("/var/cache/apt/archives", os.path.basename(i)))
        self.copy_finish(appname)
        self.counter -= 1

    #
    # dbus接口：设置是否正在进行关键更新的状态
    #
    @dbus.service.method(INTERFACE, in_signature='b', out_signature='b', sender_keyword='sender')
    def set_important_status(self, status, sender=None):
        self.counter += 1
        try:
            self.importantwork = status
        except:
            self.counter -= 1
            return False
        self.counter -= 1
        return True

    #
    # dbus接口：获取是否正在关键更新的状态
    #
    @dbus.service.method(INTERFACE, out_signature='b', sender_keyword='sender')
    def get_important_status(self, sender=None):
        return self.importantwork

    #
    # dbus接口：安装本地deb包
    #
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    def install_debfile(self, path, sender=None):
        # print("####install deb file: ", path)
        logger.info("开始安装----- %s -----", path)
        self.counter += 1
        self.efi_status = self.get_efi_status()
        self.set_efi_status(True)
            # print("send auth signal")
        #阶段一：检查路径是否存在
        path = str(path).strip().replace("\n", "")  # 防止路径传入\n和空格
        try:
            debfile = DebPackage(path,self.aptHandler.cache)
        except Exception as e:
            self.send_apt_status(path,90,str(e),False)
            logger.error("安装状态：失败 - 路径：" + path + " - 错误码：(90) - 详情：该路径不存在")
            return False
        # 阶段二：调用apt模块解析依赖并获取依赖
        kum = {"appname": debfile._sections["Package"],
               "version": debfile._sections["Version"],
               "description": debfile._sections["Description"],
               "icon": "",
               "keyword": str(2)}
        try:
            if not debfile.check(True):  #用于检查本地包是否可以安装
                message = "安装本地包无法获取依赖，或者与其它包存在冲突"
                self.send_apt_status(debfile.pkgname, 99, message, False)
                msg = {"statue": "Fail", "errorcode": str(99), "error_message": message}  # 记录结果写入数据库
                self.insert_deb_message(kum, msg)
                logger.error("安装状态：失败 - 路径：" + path + " - 错误码：(99) - 详情：" + message)
                return False
            else:
                self.aptHandler.cache.commit() #可以安装时，直接commit提交安装任务，因为依赖包已经被mark_install()
        except Exception as e:
            msg = {"statue": "Fail", "errorcode": str(98), "error_message": str(e)}  # 记录结果写入数据库
            self.insert_deb_message(kum, msg)
            self.send_apt_status(debfile.pkgname, 98, str(e), False)
            logger.error("安装状态：失败 - 路径：" + path + " - 错误码：(98) - 详情："+ str(e))
            return False
        # 阶段三：安装软件包
        # res = debfile.install(AptProcess(self, pkgName, AppActions.INSTALL_ONE,1,pkgName))
        try:
            res = debfile.install()
            if res: #安装失败
                try:
                    self.aptHandler.init_cache()
                    pkg = self.aptHandler.get_pkg_by_name(kum["appname"])
                    if pkg.is_installed:
                        pkg.mark_delete(True, True)  # 安装时失败后需要直接卸载防止apt出现依赖问题
                        self.aptHandler.cache.commit(None, None)
                except:
                    pass
                msg = {"statue": "Fail", "errorcode": str(97), "error_message": "deb包存在问题导致安装失败"}  #记录结果写入数据库
                self.insert_deb_message(kum, msg)
                self.send_apt_status(kum["appname"], 97, "deb包存在问题导致安装失败", False)
                logger.error("安装状态：失败 - 路径：" + path + " - 错误码：(97) - 详情：deb包存在问题导致安装失败")
                return False
            else: #安装成功
                msg = {"statue": "Success", "errorcode": str(0), "error_message": ""}  #记录结果写入数据库
                self.insert_deb_message(kum, msg)
                self.send_apt_status(kum["appname"], 100, AppActions.INSTALL_DEBFILE)
                logger.info("安装状态：成功 - 路径：" + path)
        except apt_pkg.Error as error:
            msg = {"statue": "Fail", "errorcode": str(97) ,"error_message": str(error)}  #记录结果写入数据库
            self.insert_deb_message(kum, msg)
            check_apt_lock() #dpkg、apt解锁
            self.send_apt_status(kum["appname"], 96, str(error), False)
            logger.info("安装状态：失败 - 路径：" + path + " - 错误码：(96) - 详情：%s"%str(error))
        logger.info("安装结束----- %s -----"%path)
        return True

    #
    # dbus接口：卸载软件包
    #
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    def remove(self, pkgName, sender=None):
        # print("####remove: ", pkgName)
        # 阶段一：初始化模块，获取软件信息
        logger.info("开始卸载----- %s -----", pkgName)
        self.counter += 1
        self.efi_status = self.get_efi_status()
        self.set_efi_status(True)
        self.aptHandler.init_cache()
        pkgName = str(pkgName).strip().replace("\n","")
        try:
            pkg = self.aptHandler.get_pkg_by_name(pkgName)
        except Exception as e:
            self.send_apt_status(pkgName, 95, str(e), False)
            logger.error("卸载状态：失败 - 包名：" + pkgName + " - 错误码：(95) - 详情：%s" % str(e))
            return False
        if pkg.is_installed is False:
            error_message = "%s未安装"%pkg
            self.send_apt_status(pkgName, 94, error_message, False)
            logger.error("卸载状态：失败 - 包名：" + pkgName + " - 错误码：(94) - 详情：软件包未安装")
            return False
        # 阶段二：开始卸载软件包
        pkg.mark_delete(True, True)
        try:
            self.aptHandler.cache.commit(None, None)
        except apt.cache.LockFailedException as e:
            self.send_apt_status(pkgName, 93, str(e), False)
            logger.error("卸载状态：失败 - 包名：" + pkgName + " - 错误码：(93) - 详情：%s"%str(e))
            # raise WorkitemError(3, "package manager is running.")
        except Exception as e:
            os.system("dpkg --configure -a")
            self.send_apt_status(pkgName, 92, str(e), False)
            logger.error("卸载状态：失败 - 包名：" + pkgName + " - 错误码：(92) - 详情：%s"%str(e))
        else:
            self.send_apt_status(pkgName,100,AppActions.REMOVE_DEBFILE)
        logger.info("卸载结束----- %s -----", pkgName)
        return True

    #
    # dbus接口：开启定时器去检测当前dbus正在被使用，没有被使用后立刻退出
    #
    @dbus.service.method(INTERFACE)
    def regular_check(self):
        exit_timer = threading.Timer(5, self.check_counter)
        exit_timer.start()

    #-------------------------------------dbus signals----------------------------------------
    #
    # dbus信号：用于发送apt下载时进度信息
    #
    @dbus.service.signal(INTERFACE, signature='sa{sv}')
    def kum_fetch_signal(self, type_msg, msg):
        pass

    #
    # dbus信号：用于发送apt安装时进度信息
    #
    @dbus.service.signal(INTERFACE, signature='sa{sv}')
    def kum_apt_signal(self, type_msg, msg):
        pass

    #
    #dbus信号：用于拷贝包到/var/cache/apt/archives的完成信号
    #
    @dbus.service.signal(INTERFACE,signature="s")
    def copy_finish(self,appname):
        pass

    #
    # dbus信号：发送重要更新包列表信息
    #
    @dbus.service.signal(INTERFACE, signature='a{sv}asasasassb')
    def important_app_message_signal(self, msg, url_list, name_list, size_list, fullname_list, all_size, depends_status):
        pass

    #
    # dbus信号：发送关键更新包列表信息
    #
    @dbus.service.signal(INTERFACE, signature='a{sv}')
    def crucial_app_message_signal(self, msg):
        pass

    #
    # dbus信号：通知前端获取重要更新信息获取完成的dbus信号
    #
    @dbus.service.signal(INTERFACE, signature='s')
    def get_message_finished_signal(self, number):
        pass

    #
    # dbus信号：通知前端数据库写入完成后的dbus信号，用于实时刷新历史更新界面
    #
    @dbus.service.signal(INTERFACE, signature='ss')
    def update_sqlite_signal(self, appname, time):
        pass
